/*
 * Sparse.hpp
 *
 *  Created on: 23 Oct 2013
 *      Author: np183
 */

#ifndef SPARSE_HPP_
#define SPARSE_HPP_

#include <iostream>
#include <string>

class Cell;

typedef unsigned int sparse_size;

typedef enum SolutionType
{
    twodarray,
    onedarray,
    map,
    linkedlist,
    other
} SolutionType;

class Sparse {
public:
	// In general in this assessment you should read two-dimensional arrays as
	// follows:
	// 0 1 2 3
	// 4 5 6 7
	// This is an array that has width 4 and height 2.
	// In location (0,0) it has the value 0
	// In location (3,1) it has the value 7
	// The location with the value 6 has the x-coordinate 2 and
	// the y-coordinate 1.

	// Initialize an empty Sparse matrix
	Sparse();
	// Initialize a Sparse matrix with the data in the two dimensional array.
	// For example: on input 2 4 {{ 0,1,2,3},{4,5,6,7}} (as above)
    	// You should have a Sparse matrix with four rows and two columns
    	// The range for rows should be 0-3 and the range for columns should
    	// be 0-1.
    	// The entry in location 0,0 should be 0, the entry in location 0,1
    	// should be 1, ...
	Sparse(sparse_size width, sparse_size height, int** array);

	// Clean your mess.
	~Sparse();

	// Add a two dimensional array with given dimensions
	// starting in location (x,y).
	// The location (x,y) should already be in the sparse matrix.
	// Otherwise return false and leave the sparse matrix unmodified.
	// Overwrite data that already exists in the sparse matrix in the positions
	// covered by the array.
	// The array has dimensions given by width and height.

	// For example, if we want to add a (2,2) array in the local (4,4)
	// of a sparse matrix that has data in locations (4,4) and (5,5) (and
	// not in (4,5) and (5,4))
	// then the data in locations (4,4) and (5,5) should be
	// overwritten and new locations should be added for (4,5) and (5,4)
	bool addArrayOverwrite(sparse_size width , sparse_size height, int** array, sparse_size x, sparse_size y);

	// Similar to add array but the information is given by
	// a sparse matrix.
	// The location (x,y) should already be in the sparse matrix.
	// Existing data should be overwritten.
	bool addSparseOverwrite(const Sparse&, sparse_size x, sparse_size y);

	// Add a two dimensional array with given dimensions starting
	// in location (x,y)
	// The location (x,y) must exist as before.
	// Locations (x',y') where y' >= y should be shifted
	// to (x', y'+height)
	// For example, adding the array
	// 350 384
	//      25
	// in position (1,1)
	// to the sparse matrix
	//
	// 200    120
	//     21  33
	//  97 45
	//
	// will yield a new sparse matrix that looks as follows:
	//
	// 200     120
	//     350 384
	//          25
	//      21  33
	//   97 45
	//
	bool addArrayPushDown(sparse_size width,sparse_size height, int** array, sparse_size x,sparse_size y);

	// Similar to PushDown but this time pushing right (i.e., increasing
	// the x-coordinates of existing rows)
	bool addArrayPushRight(sparse_size width, sparse_size height, int** array, sparse_size x,sparse_size y);

	// Insert the value in location (x,y).
	// If location (x,y) exists overwrite it (and return true).
	// If location (x,y) does not exist create and add it (and return false)
	bool insert(sparse_size x, sparse_size y, int val);

	// Does location (x,y) exist
	bool exists(sparse_size x,sparse_size y) const;

	// What is the next location in the x column (i.e., y increase)?
	// If (x,y) does not exist return false
	// If (x,y) does exist and y is the last thing
	// on the x-th column return false
	// If (x,y) does exist and there is something below y
	// update y to its number and return true.
	bool nextRow(sparse_size x, sparse_size& y) const;

	// Similar but going up (i.e., y decrease).
	bool prevRow(sparse_size, sparse_size&) const;

	// Similar going right (i.e., x-coord increasing)
	bool nextCol(sparse_size&, sparse_size) const;

	// Similar going left (i.e., x-coord decreasing)
	bool prevCol(sparse_size&, sparse_size) const;

	// Update the value to the entry in location (x,y) if exists.
	// If does not exist return false otherwise true.
	bool value(sparse_size x,sparse_size y, int& value) const;

	// Create a string that represents the contents of the matrix with location (0,0) 
	// on the
	// top left corner and location (max_x,max_y) on the bottom right corner.
	// Newlines represent the passage from row to row
	// Empty locations should be printed as spaces.
	// The matrix should be well formed (i.e., the space taken by each entry (including emmpty
	// entries) should be the same.
	// The width associated with every entry should be the maximal width required
	// by an entry that ever was inserted into this object
	// You should be able to read columns by looking on the printout of this string.
	std::string toString() const;

	// Output the string version of the sparse to the ostream.
	friend std::ostream& operator<<(std::ostream&, const Sparse&);

	SolutionType whatDidIImplement() const;
private:
	// Complete this
};


#endif /* SPARSE_HPP_ */
